/* $Id: tenm_object.c,v 1.34 2002/05/18 04:15:43 oohara Exp $ */

#include <stdio.h>
/* malloc */
#include <stdlib.h>

#include "tenm_collision.h"
#include "tenm_object.h"

tenm_mass *
tenm_mass_new(int number_primitive, tenm_primitive **p)
{
  tenm_mass *temp;

  /* sanity check */
  if (number_primitive <= 0)
    return NULL;

  temp = (tenm_mass *) malloc(sizeof(tenm_mass));
  if (temp == NULL)
  {
    fprintf(stderr, "tenm_mass_new: malloc failed\n");
    return NULL;
  }

  temp->n = number_primitive;
  temp->p = p;
  
  return temp;
}

void
tenm_mass_delete(tenm_mass *p)
{
  int i;

  if (p == NULL)
    return;

  for (i = 0; i < p->n; i++)
    if (p->p[i] != NULL)
      (p->p[i]->delete)(p->p[i]);
  free(p->p);
  free(p);
}


tenm_object *
tenm_object_new(const char *name, int attr, int hit_mask,
                int hit_point, double x, double y,
                int number_count, int *count,
                int number_count_d, double *count_d,
                int number_primitive, tenm_primitive **p,
                int (*move)(tenm_object *, double),
                int (*hit)(tenm_object *, tenm_object *),
                int (*act)(tenm_object *, const tenm_object *),
                int (*draw)(tenm_object *, int))
{
  tenm_object *temp;

  temp = (tenm_object *) malloc(sizeof(tenm_object));
  if (temp == NULL)
  {
    fprintf(stderr, "tenm_object_new: malloc failed\n");
    return NULL;
  }
  temp->mass = NULL;
  if (number_primitive >= 1)
  {
    temp->mass = tenm_mass_new(number_primitive, p);
    if (temp->mass == NULL)
    {
      fprintf(stderr, "tenm_object_new: tenm_mass_new failed\n");
      free(temp);
      return NULL;
    }
  }
  
  temp->table_index = -1;
  temp->name = name;
  temp->attr = attr;
  temp->hit_mask = hit_mask;
  temp->hit_point = hit_point;
  temp->x = x;
  temp->y = y;
  temp->n = number_count;
  temp->count = count;
  temp->n_d = number_count_d;
  temp->count_d = count_d;
  temp->move = (int (*)(void *, double)) move;
  temp->hit = (int (*)(void *, void *)) hit;
  temp->act = (int (*)(void *, const void *)) act;
  temp->draw = (int (*)(void *, int)) draw;

  return temp;
}

void
tenm_object_delete(tenm_object *p)
{
  if (p == NULL)
    return;
  tenm_mass_delete(p->mass);
  if (p->count_d != NULL)
    free(p->count_d);
  if (p->count != NULL)
    free(p->count);
  free(p);
}

/* return 1 (true) or 0 (false) */
int
tenm_collided_mass(tenm_mass *p, tenm_mass *q)
{
  int i;
  int j;
  for (i = 0; i < p->n; i++)
    for (j = 0; j < q->n; j++)
      if (tenm_collided_primitive(p->p[i], q->p[j]))
        return 1;
  return 0;
}

int
tenm_move_mass(tenm_mass *p, double dx, double dy)
{
  int i;
  for (i = 0; i < p->n; i++)
    tenm_move_primitive(p->p[i], dx, dy);
  return 0;
}
